@@ -25,7 +25,8 @@
  *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
-  
+
+#include "pcm_local.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -80,11 +81,16 @@ static void snoop_areas(snd_pcm_direct_t *dsnoop,
 {
 	unsigned int chn, schn, channels;
 	snd_pcm_format_t format;
+	unsigned int fbytes;
+	int pwidth;
 
 	channels = dsnoop->channels;
 	format = dsnoop->shmptr->s.format;
 	if (dsnoop->interleaved) {
-		unsigned int fbytes = snd_pcm_format_physical_width(format) / 8;
+		pwidth = snd_pcm_format_physical_width(format);
+		if (pwidth < 0)
+			return;
+		fbytes = pwidth / 8;
 		memcpy(((char *)dst_areas[0].addr) + (dst_ofs * channels * fbytes),
 		       ((char *)src_areas[0].addr) + (src_ofs * channels * fbytes),
 		       size * channels * fbytes);
@@ -105,7 +111,7 @@ static void snd_pcm_dsnoop_sync_area(snd_pcm_t *pcm, snd_pcm_uframes_t slave_hw_
 	snd_pcm_uframes_t hw_ptr = dsnoop->hw_ptr;
 	snd_pcm_uframes_t transfer;
 	const snd_pcm_channel_area_t *src_areas, *dst_areas;
-	
+
 	/* add sample areas here */
 	dst_areas = snd_pcm_mmap_areas(pcm);
 	src_areas = snd_pcm_mmap_areas(dsnoop->spcm);
@@ -118,7 +124,7 @@ static void snd_pcm_dsnoop_sync_area(snd_pcm_t *pcm, snd_pcm_uframes_t slave_hw_
 		size -= transfer;
 		snoop_areas(dsnoop, src_areas, dst_areas, slave_hw_ptr, hw_ptr, transfer);
 		slave_hw_ptr += transfer;
-	 	slave_hw_ptr %= dsnoop->slave_buffer_size;
+		slave_hw_ptr %= dsnoop->slave_buffer_size;
 		hw_ptr += transfer;
 		hw_ptr %= pcm->buffer_size;
 	}
@@ -134,24 +140,14 @@ static int snd_pcm_dsnoop_sync_ptr(snd_pcm_t *pcm)
 	snd_pcm_sframes_t diff;
 	int err;
 
-	switch (snd_pcm_state(dsnoop->spcm)) {
-	case SND_PCM_STATE_DISCONNECTED:
-		dsnoop->state = SNDRV_PCM_STATE_DISCONNECTED;
-		return -ENODEV;
-	case SND_PCM_STATE_XRUN:
-		if ((err = snd_pcm_direct_slave_recover(dsnoop)) < 0)
-			return err;
-		break;
-	default:
-		break;
-	}
-	if (snd_pcm_direct_client_chk_xrun(dsnoop, pcm))
-		return -EPIPE;
 	if (dsnoop->slowptr)
 		snd_pcm_hwsync(dsnoop->spcm);
 	old_slave_hw_ptr = dsnoop->slave_hw_ptr;
 	snoop_timestamp(pcm);
 	slave_hw_ptr = dsnoop->slave_hw_ptr;
+	err = snd_pcm_direct_check_xrun(dsnoop, pcm);
+	if (err < 0)
+		return err;
 	diff = pcm_frame_diff(slave_hw_ptr, old_slave_hw_ptr, dsnoop->slave_boundary);
 	if (diff == 0)		/* fast path */
 		return 0;
@@ -193,6 +189,7 @@ static int snd_pcm_dsnoop_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 	snd_pcm_status(dsnoop->spcm, status);
 	state = snd_pcm_state(dsnoop->spcm);
 	status->state = state == SND_PCM_STATE_RUNNING ? dsnoop->state : state;
+	status->hw_ptr = *pcm->hw.ptr; /* boundary may be different */
 	status->appl_ptr = *pcm->appl.ptr; /* slave PCM doesn't set appl_ptr */
 	status->trigger_tstamp = dsnoop->trigger_tstamp;
 	status->avail = snd_pcm_mmap_capture_avail(pcm);
@@ -205,22 +202,8 @@ static int snd_pcm_dsnoop_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 static snd_pcm_state_t snd_pcm_dsnoop_state(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
-	int err;
-	snd_pcm_state_t state;
-	state = snd_pcm_state(dsnoop->spcm);
-	switch (state) {
-	case SND_PCM_STATE_SUSPENDED:
-	case SND_PCM_STATE_DISCONNECTED:
-		dsnoop->state = state;
-		return state;
-	case SND_PCM_STATE_XRUN:
-		if ((err = snd_pcm_direct_slave_recover(dsnoop)) < 0)
-			return err;
-		break;
-	default:
-		break;
-	}
-	snd_pcm_direct_client_chk_xrun(dsnoop, pcm);
+
+	snd_pcm_direct_check_xrun(dsnoop, pcm);
 	return dsnoop->state;
 }
 
@@ -228,7 +211,7 @@ static int snd_pcm_dsnoop_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 {
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
 	int err;
-	
+
 	switch(dsnoop->state) {
 	case SNDRV_PCM_STATE_DRAINING:
 	case SNDRV_PCM_STATE_RUNNING:
@@ -238,7 +221,7 @@ static int snd_pcm_dsnoop_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 		/* Fall through */
 	case SNDRV_PCM_STATE_PREPARED:
 	case SNDRV_PCM_STATE_SUSPENDED:
-		*delayp = snd_pcm_mmap_capture_avail(pcm);
+		*delayp = snd_pcm_mmap_capture_delay(pcm);
 		return 0;
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
@@ -274,8 +257,7 @@ static int snd_pcm_dsnoop_reset(snd_pcm_t *pcm)
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
 	dsnoop->hw_ptr %= pcm->period_size;
 	dsnoop->appl_ptr = dsnoop->hw_ptr;
-	dsnoop->slave_appl_ptr = dsnoop->slave_hw_ptr;
-	snd_pcm_direct_reset_slave_ptr(pcm, dsnoop);
+	snd_pcm_direct_reset_slave_ptr(pcm, dsnoop, dsnoop->slave_hw_ptr);
 	return 0;
 }
 
@@ -288,8 +270,7 @@ static int snd_pcm_dsnoop_start(snd_pcm_t *pcm)
 		return -EBADFD;
 	snd_pcm_hwsync(dsnoop->spcm);
 	snoop_timestamp(pcm);
-	dsnoop->slave_appl_ptr = dsnoop->slave_hw_ptr;
-	snd_pcm_direct_reset_slave_ptr(pcm, dsnoop);
+	snd_pcm_direct_reset_slave_ptr(pcm, dsnoop, dsnoop->slave_hw_ptr);
 	err = snd_timer_start(dsnoop->timer);
 	if (err < 0)
 		return err;
@@ -326,7 +307,7 @@ static int __snd_pcm_dsnoop_drain(snd_pcm_t *pcm)
 			break;
 		if (pcm->mode & SND_PCM_NONBLOCK)
 			return -EAGAIN;
-		__snd_pcm_wait_in_lock(pcm, -1);
+		__snd_pcm_wait_in_lock(pcm, SND_PCM_WAIT_DRAIN);
 	}
 	pcm->stop_threshold = stop_threshold;
 	return snd_pcm_dsnoop_drop(pcm);
@@ -397,10 +378,10 @@ static int snd_pcm_dsnoop_close(snd_pcm_t *pcm)
 		snd_timer_close(dsnoop->timer);
 	snd_pcm_direct_semaphore_down(dsnoop, DIRECT_IPC_SEM_CLIENT);
 	snd_pcm_close(dsnoop->spcm);
- 	if (dsnoop->server)
- 		snd_pcm_direct_server_discard(dsnoop);
- 	if (dsnoop->client)
- 		snd_pcm_direct_client_discard(dsnoop);
+	if (dsnoop->server)
+		snd_pcm_direct_server_discard(dsnoop);
+	if (dsnoop->client)
+		snd_pcm_direct_client_discard(dsnoop);
 	if (snd_pcm_direct_shm_discard(dsnoop)) {
 		if (snd_pcm_direct_semaphore_discard(dsnoop))
 			snd_pcm_direct_semaphore_final(dsnoop, DIRECT_IPC_SEM_CLIENT);
@@ -419,18 +400,9 @@ static snd_pcm_sframes_t snd_pcm_dsnoop_mmap_commit(snd_pcm_t *pcm,
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
 	int err;
 
-	switch (snd_pcm_state(dsnoop->spcm)) {
-	case SND_PCM_STATE_XRUN:
-		if ((err = snd_pcm_direct_slave_recover(dsnoop)) < 0)
-			return err;
-		break;
-	case SND_PCM_STATE_SUSPENDED:
-		return -ESTRPIPE;
-	default:
-		break;
-	}
-	if (snd_pcm_direct_client_chk_xrun(dsnoop, pcm))
-		return -EPIPE;
+	err = snd_pcm_direct_check_xrun(dsnoop, pcm);
+	if (err < 0)
+		return err;
 	if (dsnoop->state == SND_PCM_STATE_RUNNING) {
 		err = snd_pcm_dsnoop_sync_ptr(pcm);
 		if (err < 0)
@@ -447,7 +419,7 @@ static snd_pcm_sframes_t snd_pcm_dsnoop_avail_update(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
 	int err;
-	
+
 	if (dsnoop->state == SND_PCM_STATE_RUNNING) {
 		err = snd_pcm_dsnoop_sync_ptr(pcm);
 		if (err < 0)
@@ -466,7 +438,7 @@ static int snd_pcm_dsnoop_htimestamp(snd_pcm_t *pcm,
 	snd_pcm_direct_t *dsnoop = pcm->private_data;
 	snd_pcm_uframes_t avail1;
 	int ok = 0;
-	
+
 	while (1) {
 		if (dsnoop->state == SND_PCM_STATE_RUNNING ||
 		    dsnoop->state == SND_PCM_STATE_DRAINING)
@@ -571,7 +543,7 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 	assert(pcmp);
 
 	if (stream != SND_PCM_STREAM_CAPTURE) {
-		SNDERR("The dsnoop plugin supports only capture stream");
+		snd_error(PCM, "The dsnoop plugin supports only capture stream");
 		return -EINVAL;
 	}
 
@@ -597,27 +569,27 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 		ret = snd_pcm_open_slave(&spcm, root, sconf, stream,
 					 mode | SND_PCM_NONBLOCK, NULL);
 		if (ret < 0) {
-			SNDERR("unable to open slave");
+			snd_error(PCM, "unable to open slave");
 			goto _err;
 		}
-	
+
 		if (snd_pcm_type(spcm) != SND_PCM_TYPE_HW) {
-			SNDERR("dsnoop plugin can be only connected to hw plugin");
+			snd_error(PCM, "dsnoop plugin can be only connected to hw plugin");
 			goto _err;
 		}
-		
+
 		ret = snd_pcm_direct_initialize_slave(dsnoop, spcm, params);
 		if (ret < 0) {
-			SNDERR("unable to initialize slave");
+			snd_error(PCM, "unable to initialize slave");
 			goto _err;
 		}
 
 		dsnoop->spcm = spcm;
-		
+
 		if (dsnoop->shmptr->use_server) {
 			ret = snd_pcm_direct_server_create(dsnoop);
 			if (ret < 0) {
-				SNDERR("unable to create server");
+				snd_error(PCM, "unable to create server");
 				goto _err;
 			}
 		}
@@ -629,10 +601,10 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 			snd_pcm_direct_semaphore_up(dsnoop, DIRECT_IPC_SEM_CLIENT);
 			ret = snd_pcm_direct_client_connect(dsnoop);
 			if (ret < 0) {
-				SNDERR("unable to connect client");
+				snd_error(PCM, "unable to connect client");
 				goto _err_nosem;
 			}
-			
+
 			snd_pcm_direct_semaphore_down(dsnoop, DIRECT_IPC_SEM_CLIENT);
 
 			ret = snd_pcm_direct_open_secondary_client(&spcm, dsnoop, "dsnoop_client");
@@ -652,18 +624,18 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 					first_instance = 1;
 					goto retry;
 				}
-				SNDERR("unable to open slave");
+				snd_error(PCM, "unable to open slave");
 				goto _err;
 			}
 			if (snd_pcm_type(spcm) != SND_PCM_TYPE_HW) {
-				SNDERR("dsnoop plugin can be only connected to hw plugin");
+				snd_error(PCM, "dsnoop plugin can be only connected to hw plugin");
 				ret = -EINVAL;
 				goto _err;
 			}
-		
+
 			ret = snd_pcm_direct_initialize_secondary_slave(dsnoop, spcm, params);
 			if (ret < 0) {
-				SNDERR("unable to initialize slave");
+				snd_error(PCM, "unable to initialize slave");
 				goto _err;
 			}
 		}
@@ -673,7 +645,7 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 
 	ret = snd_pcm_direct_initialize_poll_fd(dsnoop);
 	if (ret < 0) {
-		SNDERR("unable to initialize poll_fd");
+		snd_error(PCM, "unable to initialize poll_fd");
 		goto _err;
 	}
 
@@ -683,17 +655,17 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 	pcm->mmap_rw = 1;
 	snd_pcm_set_hw_ptr(pcm, &dsnoop->hw_ptr, -1, 0);
 	snd_pcm_set_appl_ptr(pcm, &dsnoop->appl_ptr, -1, 0);
-	
+
 	if (dsnoop->channels == UINT_MAX)
 		dsnoop->channels = dsnoop->shmptr->s.channels;
-	
+
 	snd_pcm_direct_semaphore_up(dsnoop, DIRECT_IPC_SEM_CLIENT);
 
 	*pcmp = pcm;
 	return 0;
-	
+
  _err:
- 	if (dsnoop->timer)
+	if (dsnoop->timer)
 		snd_timer_close(dsnoop->timer);
 	if (dsnoop->server)
 		snd_pcm_direct_server_discard(dsnoop);
@@ -732,7 +704,7 @@ pcm.name {
 	ipc_perm INT		# IPC permissions (octal, default 0600)
 	hw_ptr_alignment STR	# Slave application and hw pointer alignment type
 		# STR can be one of the below strings :
-		# no
+		# no (or off)
 		# roundup
 		# rounddown
 		# auto (default)
@@ -837,7 +809,7 @@ int _snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 	if (err < 0)
 		return err;
 
-	/* set a reasonable default */  
+	/* set a reasonable default */
 	if (psize == -1 && params.period_time == -1)
 		params.period_time = 125000;    /* 0.125 seconds */
 
